/**
 * 
 */
package com.jujunie.service.consolidate;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import com.jujunie.service.consolidate.ConsolidateSelector.ValueType;

/**
 * Represent a selector list
 * @author Julien BETI
 * @since 0.06.01
 * @version $Revision $
 */
public class ConsolidateSelectorsList implements Serializable, java.util.Set<ConsolidateSelector> {
    
    /** serialVersionUID */
    private static final long serialVersionUID = 4308150183535279171L;
    /** Logger */
    //private static final Log log = LogFactory.getLog(ConsolidateSelectorsList.class);
    /** Backend */
    private HashSet<ConsolidateSelector> data = new HashSet<ConsolidateSelector>();

    /* (non-Javadoc)
     * @see java.util.Set#add(java.lang.Object)
     */
    public boolean add(ConsolidateSelector o) {
        return this.data.add(o);
    }

    /* (non-Javadoc)
     * @see java.util.Set#addAll(java.util.Collection)
     */
    public boolean addAll(Collection<? extends ConsolidateSelector> c) {
        return this.data.addAll(c);
    }

    /* (non-Javadoc)
     * @see java.util.Set#clear()
     */
    public void clear() {
        this.data.clear();
        
    }

    /* (non-Javadoc)
     * @see java.util.Set#contains(java.lang.Object)
     */
    public boolean contains(Object o) {
        //log.debug("Requesting contains...");
        if (!(o instanceof ConsolidateSelector)) {
            //log.debug("Not ConsolidateSelector instance => FALSE");
            return false;
        }
        ConsolidateSelector axisSelector = (ConsolidateSelector) o;
        if (axisSelector.getType() != ValueType.DATE_RANGE) {
            // Note date range => exact match
            //log.debug("Not DATE_RANGE instance => BACKEND");
            return this.data.contains(axisSelector);
        } else {
            // Date range
            //log.debug("DATE_RANGE");
            for (ConsolidateSelector current : this.data) {
                //log.debug(current);
                // If the current is not date range, continue
                if (current.getType() != ValueType.DATE_RANGE) {
                    //log.debug("Current Not DATE_RANGE");
                    continue;
                }
                //log.debug("Current DATE_RANGE");
                // Both are date ranges :). Contains if and only if current included in other
                if (current.getDateStartValue().after(axisSelector.getDateStartValue())
                        && current.getDateEndValue().before(axisSelector.getDateEndValue())) {
                    // If found, return true
                    //log.debug("Match with axisSelector: " + axisSelector);
                    return true;
                }
            }
            //log.debug("Not found => FALSE");
            return false;
        }
    }

    /* (non-Javadoc)
     * @see java.util.Set#containsAll(java.util.Collection)
     */
    public boolean containsAll(Collection<?> c) {
        boolean res = true;
        for (Object current : c) {
            res = res && this.contains(current);
            if (!res) {
                break;
            }
        }
        return res;
    }

    /* (non-Javadoc)
     * @see java.util.Set#isEmpty()
     */
    public boolean isEmpty() {
        return this.data.isEmpty();
    }

    /* (non-Javadoc)
     * @see java.util.Set#iterator()
     */
    public Iterator<ConsolidateSelector> iterator() {
        return this.data.iterator();
    }

    /* (non-Javadoc)
     * @see java.util.Set#remove(java.lang.Object)
     */
    public boolean remove(Object o) {
        return this.data.remove(o);
    }

    /* (non-Javadoc)
     * @see java.util.Set#removeAll(java.util.Collection)
     */
    public boolean removeAll(Collection<?> c) {
        return this.data.removeAll(c);
    }

    /* (non-Javadoc)
     * @see java.util.Set#retainAll(java.util.Collection)
     */
    public boolean retainAll(Collection<?> c) {
        return this.data.retainAll(c);
    }

    /* (non-Javadoc)
     * @see java.util.Set#size()
     */
    public int size() {
        return this.data.size();
    }

    /* (non-Javadoc)
     * @see java.util.Set#toArray()
     */
    public Object[] toArray() {
        return this.data.toArray();
    }

    /* (non-Javadoc)
     * @see java.util.Set#toArray(T[])
     */
    public <T> T[] toArray(T[] a) {
        return this.data.toArray(a);
    }

}
