/**
 * 
 */
package com.jujunie.service.consolidate;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/**
 * Represents a consolidated axis
 * @author Julien BETI
 * @since 0.06.01
 * @version $Revision$
 */
public class ConsolidatedAxis implements Serializable, Comparable<ConsolidatedAxis> {
    
    /** serialVersionUID */
    private static final long serialVersionUID = -5647841164873249654L;

    /** Types of Consolidated axis */
    public enum Type {X, Y}

    /** Type */
    private Type type = Type.X;
    /** List of childs */
    private List<ConsolidatedAxis> children = new LinkedList<ConsolidatedAxis>();
    /** Parent */
    private ConsolidatedAxis parent = null;
    /** Selector */
    private ConsolidateSelector selector = null;
    /** List of elements */
    private ArrayList<ConsolidateElement> elements = new ArrayList<ConsolidateElement>();
    /** Level */
    private int level = 0;
    
    
    /**
     * Constructor
     * @param t axis type
     * @param selector string to match selectors uppon adding item
     */
    public ConsolidatedAxis(Type t, ConsolidateSelector selector) {
        this.type = t;
        this.selector = selector;
    }
    
    ConsolidatedAxis getParent() {
        return this.parent;
    }
    
    public ConsolidateSelector getSelector() {
        return this.selector;
    }
    
    public Collection<ConsolidateSelector> getAllSelectors() {
        return this.getAllSelectors(new LinkedList<ConsolidateSelector>());
    }
    
    private Collection<ConsolidateSelector> getAllSelectors(Collection<ConsolidateSelector> c) {
        c.add(this.selector);
        if (this.parent != null) {
            return this.parent.getAllSelectors(c);
        } else {
            return c;
        }
    }
    
    void sortChildren() {
        Collections.sort(this.children);
        for (ConsolidatedAxis child : this.children) {
            child.sortChildren();
        }
    }
    
    /**
     * @return unmodifiable list of children
     */
    public List<ConsolidatedAxis> getChildren() {
        return Collections.unmodifiableList(children);
    }
    
    /**
     * @param axis axis to add
     */
    public ConsolidatedAxis addOrCreateChild(ConsolidateSelector selector) {
        // Searching
        for (ConsolidatedAxis child : this.children) {
            if (child.selector.equals(selector)) {
                return child;
            }
        }
        ConsolidatedAxis axis = new ConsolidatedAxis(this.type, selector);
        axis.parent = this;
        axis.level = this.level + 1;
        this.children.add(axis);
        return axis;
    }
    
    /**
     * @return unmodifiable list of elements
     */
    public List<ConsolidateElement> getElements() {
        return Collections.unmodifiableList(this.elements);
    }
    
    /**
     * Add an element
     * @param elt
     */
    void addElement(ConsolidateElement elt) {
        this.elements.add(elt);
    }
    
    Type getType() {
        return this.type;
    }
    
    boolean isLeaf() {
        return this.children.size() == 0;
    }

    /**
     * @return the level
     */
    int getLevel() {
        return this.level;
    }

    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(ConsolidatedAxis o) {
        if (o == null) {
            return 1;
        }
        int res = 0;
        res = this.level - o.level;
        if (res == 0) {
            res = this.selector.compareTo(o.selector);
        }
        return res;
    }    
}
