/**
 * 
 */
package com.jujunie.service.consolidate;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

import com.jujunie.service.log.Log;

/**
 * Represents a consolidated axis
 * @author Julien BETI
 * @since 0.06.01
 * @version $Revision$
 */
public class ConsolidatedTable implements Serializable {
    
    /** serialVersionUID */
    private static final long serialVersionUID = -1248176261993925460L;
    /** Logger */
    private static final Log log = Log.getInstance(ConsolidatedTable.class);
    /** X axis */
    private ConsolidatedAxis x = null;
    /** Y axis */
    private ConsolidatedAxis y = null;
    /** X axis */
    private List<ConsolidatedAxis> xAxises = null;
    /** Y axis */
    private List<ConsolidatedAxis> yAxises = null;
    /**
     * @return the x
     */
    public ConsolidatedAxis getX() {
        return this.x;
    }
    /**
     * @param x the x to set
     */
    public void setX(ConsolidatedAxis x) {
        this.x = x;
    }
    /**
     * @return the y
     */
    public ConsolidatedAxis getY() {
        return this.y;
    }
    /**
     * @param y the y to set
     */
    public void setY(ConsolidatedAxis y) {
        this.y = y;
    }
    
    public void init() {
        // Sorting
        this.x.sortChildren();
        this.y.sortChildren();
        // Initialise matrix of ConsolidateElement
        this.xAxises = getAllAxis(this.x, new LinkedList<ConsolidatedAxis>());
        this.yAxises = getAllAxis(this.y, new LinkedList<ConsolidatedAxis>());
        for (ConsolidatedAxis x : this.xAxises) {
            for (ConsolidatedAxis y : this.yAxises) {
                ConsolidateElement elt = new ConsolidateElement();
                elt.addAxis(x);
                elt.addAxis(y);
                log.debug("Element added: " + elt.getXAxis().getSelector() + " <-> " + elt.getYAxis().getSelector());
            }
        }
    }
    
    static List<ConsolidatedAxis> getAllAxis(ConsolidatedAxis xAxis, List<ConsolidatedAxis> axises) {
        for (ConsolidatedAxis axis : xAxis.getChildren()) {
            getAllAxis(axis, axises);
        }
        axises.add(xAxis);
        return axises;
    }
    
    public void processItem(ConsolidateSelectorsList selectors, double value) {
        for (ConsolidatedAxis y : this.yAxises) {
            for (ConsolidateElement elt : y.getElements()) {
                //log.debug("Processing: " + elt.getXAxis().getSelector() + " <-> " + elt.getYAxis().getSelector());
                if (elt.getXAxis().isLeaf() && elt.getYAxis().isLeaf()) {
                    if (selectors.size() == 0) {
                        // If we are leaf and I do not have selectors => Illegal state
                        throw new IllegalStateException(
                                "We are leaf axis, but you did not gave me selectors... What am I supposed to do ?");
                    } else {
                        // we are leaf and I have selectors => to be or not to be :p
                        if (selectors.containsAll(elt.getXAxis().getAllSelectors()) &&
                                selectors.containsAll(elt.getYAxis().getAllSelectors())) {
                            // be !
                            //log.debug("Leaf/Leaf + match => " + value);
                            elt.setValue(elt.getValue() + value);
                        } else {
                            // not be !
                            //log.debug("Leaf/Leaf + nomatch => 0.0");
                        }
                    }
                } else {
                    //log.debug("No Leaf => Skipping");
                }
            }
        }
    }

    public void consolidate() {
        for (ConsolidatedAxis y : this.yAxises) {
            for (ConsolidateElement elt : y.getElements()) {
                //log.debug("Processing: " + elt.getXAxis().getSelector() + " <-> " + elt.getYAxis().getSelector());
                if (elt.getXAxis().isLeaf()) {
                    if (elt.getYAxis().isLeaf()) {
                        // Nothing to do
                    } else {
                        // The sibling is not leaf ==> consolidating
                        // To do so, adding my elements value until I found a element where sibling have the same level
                        consolidate(elt, false);
                    }
                } else {
                    // I'm not leaf ==> consolidating
                    // To do so, adding sibling elements value until I found a element where I have the same level
                    consolidate(elt, true);
                }
            }
        }
    }

    /**
     * Updating the last element value by consolidating value of previous elements that have associated sibling axis
     * level exactly equal to current sibling level + 1, until sibling level equal current sibling level.
     */
    private static void consolidate(ConsolidateElement last, boolean fromY) {
        //log.debug("Consolidating");
        ConsolidateElement current;
        int currentLevel = 0;
        // From which axe I have to sum ? 
        ConsolidatedAxis from = fromY ? last.getYAxis() : last.getXAxis();
        // Getting stop level
        int stopLevel = fromY ? last.getXAxis().getLevel() : last.getYAxis().getLevel();
        //log.debug("Stop level: " + stopLevel);
        // Getting begin index
        int beginIndex = fromY ? last.getYIndex() : last.getXIndex();
        //log.debug("Begin index: " + beginIndex);
        for (ListIterator<ConsolidateElement> elts = from.getElements().listIterator(beginIndex);
            elts.hasPrevious();) {
            current = elts.previous();
            currentLevel = (fromY ? current.getXAxis().getLevel() : current.getYAxis().getLevel());
            //log.debug("Current element: " + current.getXAxis().getSelector() + " <-> " + current.getYAxis().getSelector());
            //log.debug("Current level: " + currentLevel);
            //log.debug("Current value: " + current.getValue());
            // Stopping if stopLevel
            if (stopLevel == currentLevel) {
                //log.debug("Current level == stop level => BREAK ");
                // Break !
                break;
            } else if (currentLevel == (stopLevel + 1)) {
                //log.debug("Current level == (stop level + 1) => ADD ");
                // Adding
                last.setValue(last.getValue() + current.getValue());
            } else {
                //log.debug("Current level != (stop level + 1) => SKIPPING ");
                // Intermitant level
            }
        }
        // Done :)
    }
}
