/**
 * 
 */
package com.jujunie.service.consolidate;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import com.jujunie.service.consolidate.ConsolidatedAxis.Type;

/**
 * Generate HTML from Consolidated table
 * @author Julien BETI
 * @since 1.0
 * @version $Revision $
 */
public class ConsolidatedTable2Html {
    /** Main table class */
    private String mainTableClassStyle = "";
    /** X Axis show level */
    private int xShowLevel = 100;
    /** Y Axis show level */
    private int yShowLevel = 100;
    /** Consolidated table */
    private ConsolidatedTable consolidation = null;
    
    /**
     * Constructor
     * @param t consolidated table
     */
    public ConsolidatedTable2Html(ConsolidatedTable t) {
        this.consolidation = t;
    }
    /**
     * @return the mainTableClassStyle
     */
    public String getMainTableClassStyle() {
        return this.mainTableClassStyle;
    }
    /**
     * @param mainTableClassStyle the mainTableClassStyle to set
     */
    public void setMainTableClassStyle(String mainTableClassStyle) {
        this.mainTableClassStyle = mainTableClassStyle;
    }
    /**
     * @return the xShowLevel
     */
    public int getXShowLevel() {
        return this.xShowLevel;
    }
    /**
     * @param showLevel the xShowLevel to set
     */
    public void setXShowLevel(int showLevel) {
        this.xShowLevel = showLevel;
    }
    /**
     * @return the yShowLevel
     */
    public int getYShowLevel() {
        return this.yShowLevel;
    }
    /**
     * @param showLevel the yShowLevel to set
     */
    public void setYShowLevel(int showLevel) {
        this.yShowLevel = showLevel;
    }
    public String toHTML() {
        StringBuffer res = new StringBuffer("\n<table class=\"" + this.mainTableClassStyle + "\">");
        // Generates X header
        this.generateXHeaderRow(res, Collections.singletonList(this.consolidation.getX()), true);
        this.generateYRow(res, this.consolidation.getY(), true);
        res.append("\n</table>");
        return res.toString();
    }
    
    private void generateYRow(StringBuffer res, ConsolidatedAxis y, boolean generateTr) {
        int depth = this.getDepth(y);
        String classStyle = "level_y_" + y.getLevel();
        // Rowspanning the number of leaf, plus 1 by level for totals, minus 1 as leaf does not have total :p
        int rowspan = this.getLeafNumbers(y) - 1;
        // The algorithm can return 0 for leaf...
        if (rowspan < 1) {
            rowspan = 1;
        }
        if (generateTr) {
            res.append("\n <tr>");
        }
        res.append("\n  <th class=\"" + classStyle + "\" rowspan=\"" + rowspan + "\">");
        res.append(y.getSelector().toGUIViewString());
        res.append("</th>");
        // Generate childs
        boolean nextGenerateTr = false;
        if(y.getLevel() < yShowLevel) {
            for (ConsolidatedAxis child : y.getChildren()) {
                this.generateYRow(res, child, nextGenerateTr);
                nextGenerateTr = true;
            }
            if (!y.isLeaf()) {
                res.append("\n <tr>");
                res.append("\n  <th class=\"" + classStyle + "\" colspan=\"" + depth + "\">Total</th>");
            } else {
                // nothing to do
            }
        }
        this.generateElements(res, y.getElements());
        res.append("\n </tr>");
    }
    
    private void generateElements(StringBuffer res, List<ConsolidateElement> elts) {
        String classStyle;
        double value = 0.0d;
        int xLevel = -1;
        int yLevel = -1;
        for (ConsolidateElement elt : elts) {
            // I generate only if I have to...
            xLevel = elt.getXAxis().getLevel();
            yLevel = elt.getYAxis().getLevel();
            if (xLevel <= xShowLevel && yLevel <= yShowLevel) {
                classStyle = "level_" + xLevel + '_' + yLevel;
                value = Math.round(elt.getValue() * 100)/100d;
                res.append("\n  <td class=\"" + classStyle + "\">")
                   .append(value == 0.0d ? "&nbsp;" : String.valueOf(value))
                   .append("</td>");
            }
        }
    }
    
    private void generateXHeaderRow(StringBuffer res, List<ConsolidatedAxis> xs, boolean generateFirstCell) {
        LinkedList<ConsolidatedAxis> next = new LinkedList<ConsolidatedAxis>();
        String classStyle;
        // Generating X header
        res.append("\n<tr>");
        if (generateFirstCell) {
            // First cell: covering Y headers depth
            res.append("\n<th class=\"level_0_0\" " +
                       "      rowspan=\"" + this.getDepth(this.consolidation.getX()) + "\" " +
                       "      colspan=\"" + this.getDepth(this.consolidation.getY()) + "\">Hours Worked Report</th>");
        }
        // Other cells
        for (ConsolidatedAxis x : xs) {
            classStyle = "level_x_" + x.getLevel();
            // Colspanning the number of leaf
            int colspan = this.getLeafNumbers(x) - 1;
            // The algorithm can return 0 for leaf...
            if (colspan < 1) {
                colspan = 1;
            }
            res.append("\n<th class=\"" + classStyle + "\" colspan=\"" + colspan + "\">");
            res.append(x.getSelector().toGUIViewString());
            res.append("</th>");
            if (x.getLevel() < xShowLevel) {
                next.addAll(x.getChildren());
                // Total cell, if not leaf
                if (!x.isLeaf()) {
                    res.append("\n<th class=\"" + classStyle + "\" rowspan=\"" + this.getDepth(x) + "\">Total</th>");
                }
            }
        }
        res.append("\n</tr>");
        // generating next line
        if (next.size() > 0) {
            this.generateXHeaderRow(res, next, false);
        }
    }
    
    private int getDepth(ConsolidatedAxis axis) {
        int maxDepth = (axis.getType() == Type.X ? this.xShowLevel : this.yShowLevel);
        return getDepth_aux(axis, maxDepth);
    }
    private static int getDepth_aux(ConsolidatedAxis axis, int maxDepth) {
        if (axis.isLeaf() || axis.getLevel() == maxDepth) {
            return 1;
        } else {
            return 1 + getDepth_aux(axis.getChildren().get(0), maxDepth);
        }
    }
    
    private int getLeafNumbers(ConsolidatedAxis axis) {
        int maxDepth = (axis.getType() == Type.X ? this.xShowLevel : this.yShowLevel);
        return getLeafNumbers_aux(axis, maxDepth);
    }
    private static int getLeafNumbers_aux(ConsolidatedAxis axis, int maxDepth) {
        if (axis.isLeaf() || axis.getLevel() == maxDepth) {
            return 1;
        } else {
            int sum = 0;
            for (ConsolidatedAxis child : axis.getChildren()) {
                sum += getLeafNumbers_aux(child, maxDepth);
            }
            return sum + ((axis.getChildren().size() != 0 && axis.getLevel() < maxDepth) ? 1 : 0);
        }
    }
    

}
