/*
 *==========================================
 * Cretaed from org.kolaka.jdf.util.Log :
 *
 * JDataField - the Java Data Field GUI
 *
 * This code was developped by Alban Peignier (http://www.tryphon.org/~alban/) 
 * and contributors (their names can be found in the CONTRIBUTORS file).
 *
 * Copyright (C) 2002 Alban Peignier
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *==========================================
 */
package com.jujunie.service.log;

import java.util.Map;
import java.util.TreeMap;

/**
 * Class used to log debug, info, warn and error messages.
 * It delegates the calls to an implementation of this absctract class
 * @author  Julien BETI
 */
public abstract class Log {
    
  /*
   * Instance creation
   */
   
  private static Map<String, Log> instances = new TreeMap<String, Log>();
  private static LogFactory logFactory = null;

    static {
        try{
            String logFactoryName = System.getProperty(LogFactory.class.getName());
            if(logFactoryName != null) {
            	logFactory = (LogFactory)Class.forName(logFactoryName).newInstance();
            }
		} catch(SecurityException se)  {
			se.printStackTrace(System.err);
		} catch(NullPointerException npe)  {
			npe.printStackTrace(System.err);
		} catch(IllegalArgumentException iae)  {
			iae.printStackTrace(System.err);
		} catch(ClassNotFoundException cnf)  {
			cnf.printStackTrace(System.err);
		} catch(InstantiationException ie)  {
			ie.printStackTrace(System.err);
		} catch(IllegalAccessException iae2)  {
			iae2.printStackTrace(System.err);
		}
        if (logFactory == null) {
            logFactory = new LogFactoryImpl();
        }
    }
    
  public static Log getInstance(Object o) {
    if (o == null)
      throw new IllegalArgumentException("No specified Object");
    return getInstance(o.getClass());
  }

  public static Log getInstance(Class<? extends Object> c) {
    if (c == null) {
      throw new IllegalArgumentException("No specified Class");
    }
    return getInstance(c.getName());
  }

  public static Log getInstance(String name) {
    Log instance = (Log) instances.get(name);
    if (instance == null)
      instances.put(name, instance = create(name));
    return instance;
  }

    protected static Log create(String name) {
        return logFactory.getLogger(name);
    }

  /*
   * Fields
   */
  protected String name;

  /*
   * Constructor
   */
  protected Log(String name) {
    this.name = name;
  }

  /*
   * Methods
   */
  public abstract void debug(String message);

  public abstract void debug(String message, Object toLog);

  public abstract void debug(MessageFactory heavyMessage);

  public abstract void debug(String message, Throwable t);

  public abstract void enter(String method);
  
  public abstract void exit(String method);

  public abstract void info(String message);

  public abstract void info(String message, Object toLog);

  public abstract void warn(String message);

  public abstract void error(String message, Throwable t);
  
  public abstract void error(String message);
  
  public static interface MessageFactory {
    public String createMessage();
  }
  
}
