/*
 * Created on 11 nov. 2003
 *
 */
package com.jujunie.service.web;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.jujunie.service.log.Log;
import com.jujunie.service.util.BooleanUtil;

/**
 * HttpServlet implementation.
 * The MainServlet class is supposed to be the main and only servlet
 * of a project. It is the convergence point of all request/response project.
 * @author <a href="julien@jujunie.com>julien</a>
 * @version $Revision $
 */
public class MainServlet extends HttpServlet implements WebController {

    /**
     * serialVersionUID
     */
    private static final long serialVersionUID = -4740436634868081365L;

    /**
     * Config file parameter key. The path to the configuration file must be given
     * in web.xml init-parameters using this key. The key value is "web-configuration-file"
     */
    public static final String KEY_CONFIGURATION_FILE = "web-configuration-file";
    
    /**
     * Key in servlet context to retreive the maintenance mode
     */
    private static final String KEY_MAINTENANCE_MODE = "is-maintenance-mode";
    
    /**
     * Key prefix for framework keys
     */
    private static final String KEY_PREFIX = "jujunie-framework-g4e54terg1g1ert5d1215645zert1z2f1565e1rt-";
    
    /**
     * Key in servlet context to retreive the web configuration
     */
    private static final String KEY_WEB_CONFIGURATION = KEY_PREFIX + "web-config";
    
    /**
     * Key to retreive current module in session
     */
    private static final String KEY_CURRENT_PAGE = KEY_PREFIX + "current-page";
    
    /**
     * Key to retreive command in request
     */
    private static final String KEY_COMMAND = "command";
    
    /**
     * Key to retreive page in request
     */
    private static final String KEY_PAGE = "page";
    
    /**
     * Key to retreive module in request
     */
    private static final String KEY_MODULE = "module";
    
    /**
     * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public final void doGet(HttpServletRequest arg0, HttpServletResponse arg1)
        throws ServletException, IOException {
        this.doIt(arg0, arg1);
    }

    /**
     * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public final void doPost(HttpServletRequest arg0, HttpServletResponse arg1)
        throws ServletException, IOException {
        this.doIt(arg0, arg1);
    }
    
    /**
     * Handle a request/response process
     * 
     * @param req the http request to process
     * @param resp the ouput http response
     */
    private void doIt(HttpServletRequest req, HttpServletResponse resp) {
        Log.getInstance(this).enter("doIt");
        WebConfig conf = this.getWebConfig();
        conf.getDefaultApplication().getWebTransactionHandler().begin(req);
        WebPage page;
        String commandName;
        Command command;
        CommandResult commandResult = null;
        // Getting session
        HttpSession session = req.getSession(true);
        try {
            if (this.isMaintenanceMode()) {
                try {
                    resp.setContentType("text/html");
                    PrintWriter out = resp.getWriter();
                    out.println("<html>");
                    out.println("<body>");
                    out.println("<head>");
                    out.println("<title>Application in maintenance mode</title>");
                    out.println("</head>");
                    out.println("<body>");
                    out.println("<h1>Sorry, the requested application is in maintenance</h1>");
                    out.println("</body>");
                    out.println("</html>");
                    out.flush();
                    return;
                } catch (IOException e) {
                    Log.getInstance(this).error("IOError !", e);
                }
            }
            page = this.selectPage(req, session, conf);
            Log.getInstance(this).debug("Now Looking for a command to execute");
            commandName = req.getParameter(KEY_COMMAND);
            if (commandName != null) {
                Log.getInstance(this).debug(
                        "Command name found in request : " + commandName);
                Log.getInstance(this).debug("Retreiving command in current page");
                try {
                    command = page.getCommand(commandName);
                    Log.getInstance(this).debug(
                        "Now executing : " + command.getDescription());
                    commandResult = command.execute(this, req);
                } catch (WebItemNotAvailableException e) {
                    Log.getInstance(this).error("The command " + commandName
                            + " cannot be found for page " + page.getCode(), e);
                }
                Log.getInstance(this)
                    .debug("Now processing command result");
                if (commandResult != null) {
                    try {
                        page = page.getOutput(commandResult.getOutputName());
                    } catch (WebItemNotAvailableException e) {
                        Log.getInstance(this).error(
                                "The output " + commandResult.getOutputName()
                                + " cannot be found for page " + page.getCode(), e);
                    }
                } else {
                    Log.getInstance(this).debug(
                        "CommandResult object is null => Keeping current page");
                }
            } else {
                Log.getInstance(this)
                    .debug("Cannot found a command to execute in request");

            }
            Log.getInstance(this).debug("Displaying page : " + page.getCode());
            try {
                page.getDisplayHandler().display(page, req, resp);
                session.setAttribute(KEY_CURRENT_PAGE, page);
            } catch (DisplayException e) {
                String message = "Error while displaying page '" + page.getCode() + "' : " + e.getMessage();
                Log.getInstance(this).error(message, e);
            }
            conf.getDefaultApplication().getWebTransactionHandler().end(req);
        } catch (RuntimeException error) {
             conf.getDefaultApplication().getWebTransactionHandler().onError(req, error);
             throw error;
        }
        Log.getInstance(this).exit("doIt");
    }
    
    /**
     * This method selects the current page. To do so, it
     * first look in the request for module and page attributes.
     * Otherwise, it returns the last page known in session 
     */
    private WebPage selectPage(HttpServletRequest req, HttpSession session, WebConfig conf) {
        Log.getInstance(this).enter("selectPage");
        WebPage page = null;
        String strmodule = req.getParameter(KEY_MODULE);
        String strpage = req.getParameter(KEY_PAGE);
        if(strmodule != null && strpage != null) {
            Log.getInstance(this).debug("Page and module found in request:");
            Log.getInstance(this).debug("Module: ", strmodule);
            Log.getInstance(this).debug("Page: ", strpage);
            try {
                page = conf.getDefaultApplication().getModule(strmodule).getPage(strpage);
            } catch(WebItemNotAvailableException e) {
                Log.getInstance(this).error("Cannot find page " + strpage + " in module " + 
                        strmodule + ". Continuing anyway...");
            }
            Log.getInstance(this).debug("Page successfuly retrieved from request information");
        } else {
            Log.getInstance(this).debug("Page and module NOT found in request. Looking for current page in session");
            // Retrieving current module and page
            page = (WebPage) session.getAttribute(KEY_CURRENT_PAGE);
        }
        if (page == null) {
            Log.getInstance(this).debug("No page found in session. Getting the default page"
                    + " of the default module of the default application");
            WebModule module = conf.getDefaultApplication().getDefaultModule();
            page = module.getDefaultPage();
        }
        Log.getInstance(this).debug("Selected page: ", page.getCode());
        Log.getInstance(this).exit("selectPage");
        return page;
    }

    /**
     * @see javax.servlet.Servlet#getServletInfo()
     */
    public String getServletInfo() {
        return super.getServletInfo() + " --- Jujunie's MainServlet --- ";
    }

    /**
     * @see javax.servlet.Servlet#init(javax.servlet.ServletConfig)
     */
    public void init(ServletConfig conf) throws ServletException {
        super.init(conf);
        Log.getInstance(this).enter("init");
        String configFilePath;
        this.setMaintenanceMode(false);
        Log.getInstance(this).debug("Loading the configuration file. The path should be given in servlet config");
        configFilePath = conf.getInitParameter(KEY_CONFIGURATION_FILE);
        if (configFilePath == null || configFilePath.length() == 0) {
            Log.getInstance(this).error("The path to the web configuration file cannot be found."
            + " Please check your web.xml configuration");
            this.setMaintenanceMode(true);
        } else {
            this.setWebConfigPath(configFilePath);
            try {
                this.reloadWebConfig();
            } catch (WebConfigException e) {
                Log.getInstance(this).error("The web configuration cannot be loaded : " + e.getMessage(), e);
                this.setMaintenanceMode(true);
            }
        }
        if (!this.isMaintenanceMode()) {
            Log.getInstance(this).debug("Executing init command");
            if (this.getWebConfig().getDefaultApplication().getInitCommand() != null) {
                this.getWebConfig().getDefaultApplication().getInitCommand().execute(this, conf);
            }
        }
        Log.getInstance(this).info("MainServlet initialized");
        Log.getInstance(this).exit("init");
    }
    
    /**
     * Reload the web configuration file
     * @throws WebConfigException if an error occurs while reloading configuration
     */
    public void reloadWebConfig() throws WebConfigException {
        Log.getInstance(this).enter("reloadWebConfig");
        // Retreiving the WebConfig stored in srvlet context
        WebConfig conf = this.getWebConfig();
        if (conf == null) {
            Log.getInstance(this).debug("Web configuration not found in servlet context, creating a new one");
            conf = new WebConfigXML();
            this.setWebConfig(conf);
        }
        String path = this.getWebConfigPath();
        conf.init(this.getServletContext().getResourceAsStream(path));
        Log.getInstance(this).exit("reloadWebConfig");
    }
    
    /**
     * Set the web configuration to the servlet context
     * @param conf the web configuration to store
     */
    private void setWebConfig(WebConfig conf) {
        this.getServletContext().setAttribute(KEY_WEB_CONFIGURATION, conf);
    }
    
    /**
     * Retreive the web configuration stored in servlet context
     * @return WebConfig
     */
    public WebConfig getWebConfig() {
        return (WebConfig) this.getServletContext().getAttribute(KEY_WEB_CONFIGURATION);
    }
    
    
    /**
     * Set the path to the configuration file in the servlet context
     * @param path path to the configuration file
     */
    private void setWebConfigPath(String path) {
        this.getServletContext().setAttribute(KEY_CONFIGURATION_FILE, path);
    }
    
    /**
     * @return the path to the configuration file
     */
    private String getWebConfigPath() {
        return (String) this.getServletContext().getAttribute(KEY_CONFIGURATION_FILE);
    }
    
    /**
     * The method move the application to or out to maintenance mode.
     * When the maintenance mode is activated, a simple message is written to the client. This
     * message can be configured in the web configuration file as an html file. If the web
     * configuration file is not available, the error message is directly written to the response.
     * @param isMaintenance
     */
    public void setMaintenanceMode(boolean isMaintenance) {
        if (isMaintenance) {
            Log.getInstance(this).info("Moving application to maintenance mode");
        } else {
            Log.getInstance(this).info("Moving application to normal mode");
        }
        this.getServletContext().setAttribute(KEY_MAINTENANCE_MODE,
            BooleanUtil.getBoolean(isMaintenance));
    }
    
    /**
     * @return true if the sysytem is in maintenance mode
     */
    private boolean isMaintenanceMode() {
        return BooleanUtil.getBoolean((Boolean) this.getServletContext()
            .getAttribute(KEY_MAINTENANCE_MODE));
    }

}
