/*
 * Created on Dec 21, 2003
 *
 */
package com.jujunie.service.web;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.velocity.Template;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.io.VelocityWriter;
import org.apache.velocity.servlet.VelocityServlet;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.jujunie.service.log.Log;
import com.jujunie.service.xml.InvalidXMLElementException;

/**
 * Display handler which uses Velocity. 
 * The WebPage object is pushed into the velocity context using"webPage" key.
 * The HtmlElementsProvider is pushed in the velocity context using "htmlElements" key.
 * @author julien
 * @version $Revision $
 */
public class VelocityDisplayHandlerXML extends WebElementXML implements DisplayHandlerXML {

    /** Property tag name */
    private final static String ELT_PROPERTY_NAME = "property";
    /** Code */
    private String code = "";
    /**
     * @see com.jujunie.service.web.DisplayHandlerXML#init(org.w3c.dom.Element)
     */
    public void init(Element displayHandlerNode)
    throws WebConfigException, InvalidXMLElementException {
        Log.getInstance(this).enter("init");
        this.code = super.getAttribute(displayHandlerNode, ATT_CODE, true);
        NodeList props = displayHandlerNode.getElementsByTagName(ELT_PROPERTY_NAME);
        Element currElt;
        String name, value;
        for (int i = 0; i < props.getLength(); i++) {
            currElt = (Element) props.item(i);
            name = super.getAttribute(currElt, ATT_NAME, true);
            value = super.getAttribute(currElt, ATT_VALUE, true);
            Log.getInstance(this).debug("Adding velocity property : " + name + "=" + value);
            Velocity.setProperty(name, value);
        }
        try {
	        Velocity.init();
		} catch (Exception e) {
		    String message = "An exception occurs while initialising the Velocity engine : " + e.getMessage();
            Log.getInstance(this).error(message, e);
            throw new WebConfigException(message, e);
        }
        Log.getInstance(this).exit("init");
    }

    /**
     * @see com.jujunie.service.web.DisplayHandler#display(com.jujunie.service.web.WebPage, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void display(WebPage page, HttpServletRequest req, HttpServletResponse resp)
    throws DisplayException {
        Log.getInstance(this).enter("display");
        Template tpl = null;
        // Creating display context
        DisplayContext context = new VelocityDisplayContext();
        context.set(VelocityServlet.REQUEST,  req );
        context.set(VelocityServlet.RESPONSE, resp);
        context.set("webPage", page);
        context.set("htmlElements", page.getWebApplication().getHtmlElements());
        // Initialise the view
        page.getDisplayInitialiser().initDisplay(context, req);
        try {
            //Getting the template
            tpl = Velocity.getTemplate(page.getUsedTemplate().getContainer());
            // Setting content type
            resp.setContentType(page.getContentType());
            resp.addHeader("Content-Encoding", page.getEncoding());
            // Merging template
            this.mergeTemplate(tpl, (Context) context, resp, page.getEncoding());
        } catch (ResourceNotFoundException e) {
            String message = "Error while getting velocity template : resource not found";
            Log.getInstance(this).error(message, e);
            throw new DisplayException(message, e);
        } catch (ParseErrorException e) {
            String message = "Error while getting velocity template : parse error";
            Log.getInstance(this).error(message, e);
            throw new DisplayException(message, e);
        } catch (UnsupportedEncodingException e) {
            String message = "Error while executing velocity template. The encoding "
                           + page.getEncoding() + " is not supported";
            Log.getInstance(this).error(message, e);
            throw new DisplayException(message, e);
        } catch (Exception e) {
            String message = "Error while getting velocity template : unknown error";
            Log.getInstance(this).error(message, e);
            throw new DisplayException(message, e);
        }
        Log.getInstance(this).exit("display");
    }

    /**
     *  merges the template with the context. It have been taken from the VelocityServlet
     * source code.
     *
     *  @param template template object returned by the handleRequest() method
     *  @param context  context created by the createContext() method
     *  @param response servlet reponse (use this to get the output stream or Writer
     */
    protected void mergeTemplate(Template template, Context context, HttpServletResponse response, String encoding)
        throws ResourceNotFoundException, ParseErrorException, MethodInvocationException,
        IOException, UnsupportedEncodingException, Exception {
        Log.getInstance(this).enter("mergeTemplate");
        ServletOutputStream output = response.getOutputStream();
        VelocityWriter vw = null;
        
        try {
            vw = new VelocityWriter(new OutputStreamWriter(output, encoding), 4*1024, true);
            template.merge(context, vw);
        } finally {
            if (vw != null) {
                vw.flush();
            }                
        }
        Log.getInstance(this).exit("mergeTemplate");
    }

    /**
     * @see com.jujunie.service.web.DisplayHandlerXML#getCode() 
     */    public String getCode() {
        return this.code;
    }

}
