/*
 * Created on 11 nov. 2003
 *
 */
package com.jujunie.service.web;

import com.jujunie.service.ApplicationContext;
import com.jujunie.service.web.html.HtmlElementsProvider;

/**
 * Represent an application
 * @author <a href="mailto:julien@jujunie.com">julien</a>
 */
public interface WebApplication {

    /**
     * @return true if the application is active
     */
    boolean isActive();
    
    /**
     * @return the name of the application
     */
    String getName();
    
    /**
     * @return the code of the application
     */
    String getCode();
    
    /**
     * Returns the web configuration
     * @return the web configuration
     */
    WebConfig getWebConfig();
    
    /**
     * @return the default module of this application
     */
    WebModule getDefaultModule();
    
    /**
     * @return the default template of this application
     */
    WebTemplate getDefaultTemplate();
    
    /**
     * Returns the template associated to the given code
     * @param templateCode code of the template to retreive
     * @return
     * @throws WebItemNotAvailableException
     */
    WebTemplate getTemplate(String templateName) throws WebItemNotAvailableException;
    
    /**
     * Get a webModule using a module path.
     * A module path is by example : "moduleCode1/moduleCode1.2/moduleCode1.2.5"
     * @param modulePath path of the module to retreive
     * @return the WebModule corresponding to the given path
     * @throws WebItemNotAvailableException if the requested module is not found
     */
    WebModule getModule(String modulePath) throws WebItemNotAvailableException;
    
    /**
     * @param code Property code
     * @return the Property corresponding to the given code
     */
    WebProperty getWebProperty(String code);
    
    /**
     * Returns the command that should be executed after MainServlet init
     * @return the command that should be executed after MainServlet init
     */
    InitCommand getInitCommand();
    
    /**
     * @return application context
     */
    ApplicationContext getApplicationContext();
    
    /**
     * @return the HTML elements that have been configured, loaded and cached
     */
    HtmlElementsProvider getHtmlElements();
    
    /**
     * @return the transaction handler associated to this application
     */
    WebTransactionHandler getWebTransactionHandler();
}
