/*
 * Created on 11 nov. 2003
 *
 */
package com.jujunie.service.web;

import java.io.InputStream;
import java.io.Serializable;

/**
 * This interface defines methods to access the web configuration.
 * The configuration is defined by a xml file wich is given as an input
 * stream.
 * @author <a href="mailto:julien@jujunie.com">julien</a>
 */
public interface WebConfig extends Serializable {
    
    /**
     * Initialise the web configuration using an inputStream
     * @param xmlConfig
     * @return the WebConfig instance
     * @throws WebConfigException if a WebConfiguration exception occurs
     */
    WebConfig init(InputStream xmlConfig) throws WebConfigException;
    
    /**
     * Get the display handler corresponding to the given code
     * @param code the display handler code
     * @return the display handler
     */
    DisplayHandler getDisplayHandler(String code);
    
    /**
     * Get the default display handler
     * @return the display handler
     */
    DisplayHandler getDefaultDisplayHandler();
    /**
     * Returns the default application
     * @return the default application
     */
    WebApplication getDefaultApplication();
    
}
