/*
 * Created on Nov 29, 2003
 *
 */
package com.jujunie.service.web;

import java.util.LinkedList;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.jujunie.service.xml.*;

/**
 * This class provides base services for XML implementation of WebConfig
 * @author julien
 *
 */
public class WebElementXML {
    
    /** Active attribute name */
    protected static final String ATT_ACTIVE = "active";
    
    /** Class attribute name */
    protected static final String ATT_CLASS = "class";
    
    /** Code attribute name */
    protected static final String ATT_CODE = "code";

    /** Active attribute value */
    protected static final String ATT_CONTAINER = "container";
    
    /** Content type attribute name */
    protected static final String ATT_CONTENT_TYPE = "content-type";
    
    /** Column style class attribute name */
    protected static final String ATT_COLUMN_STYLE_CLASS = "column-style-class";
    
    /** Default application attribute name */
    protected static final String ATT_DEFAULT_APP = "default-application";

    /** Default display handler attribute name */
    protected static final String ATT_DEFAULT_DI = "default-display-handler";
    
    /** Default module attribute name */
    protected static final String ATT_DEFAULT_MODULE = "default-module";
    
    /** Default page attribute name */
    protected static final String ATT_DEFAULT_PAGE = "default-page";
    
    /** Default template attribute name */
    protected static final String ATT_DEFAULT_TEMPLATE = "default-template";
    
    /** Default description attribute name */
    protected static final String ATT_DESCRIPTION = "description";
    
    /** Display hanlder attribute name */
    protected static final String ATT_DI = "display-handler";
    
    /** Encoding attribute name */
    protected static final String ATT_ENCODING = "encoding";
    
    /** Header style class attribute name */
    protected static final String ATT_HEADER_STYLE_CLASS = "header-style-class";
    
    /** Label code attribute name */
    protected static final String ATT_LABEL_CODE = "label-code";
    
    /** Module attribute name */
    protected static final String ATT_MODULE = "module";
    
    /** Name attribute name */
    protected static final String ATT_NAME = "name";
    
    /** Page attribute name */
    protected static final String ATT_PAGE = "page";
    
    /** Row manager attribute name */
    protected static final String ATT_ROW_MANAGER = "row-manager";
    
    /** Template attribute name */
    protected static final String ATT_TEMPLATE = "template";
    
    /** Style class attribute name */
    protected static final String ATT_STYLE_CLASS = "style-class";
    
    /** Active attribute value */
    protected static final String ATT_VALUE = "value";
    
    
    /**
     * @param e parent element
     * @param name child node name
     * @return the list of childs with the given name
     */
    protected NodeList getChildsNamed(Element e, String name) {
        NodeList list = e.getChildNodes();
        final LinkedList<Node> backend = new LinkedList<Node>();
        for(int i = 0; i < list.getLength(); i++) {
            if(name.equalsIgnoreCase(list.item(i).getNodeName())) {
                backend.add(list.item(i));
            }
        }
        NodeList res = new NodeList(){
        
            public Node item(int index) {
                return backend.get(index);
            }
        
            public int getLength() {
                return backend.size();
            }
        
        };;
        return res;
    }
    
    /**
     * Returns the value of the attribute with the given name.
     * If the attribute is not mandatory and not present, an empty
     * string is returned.
     * @param e the element containing the attribute
     * @param attName the name of the attribute to retreive
     * @param isMandatory specify if the requested attribute value is mandatory
     * @return the attribute value
     * @throws InvalidXMLElementException if the attribute is mandatory an,d not present
     */
    protected String getAttribute(Element e, String attName, boolean isMandatory) throws InvalidXMLElementException {
        return this.getAttribute(e, attName, isMandatory, "");
    }
  
    /**
     * Returns the value of the attribute with the given name.
     * If the attribute is not mandatory and not present, an empty
     * string is returned.
     * @param e the element containing the attribute
     * @param attName the name of the attribute to retreive
     * @param isMandatory specify if the requested attribute value is mandatory
     * @param defaultValue default value to return if the attribute is not found
     * @return the attribute value
     * @throws InvalidXMLElementException if the attribute is mandatory an,d not present
     */
    protected String getAttribute(Element e, String attName, boolean isMandatory, String defaultValue)
    throws InvalidXMLElementException {
        String attValue = e.getAttribute(attName);
        if (isMandatory) {
            if (attValue == null || attValue.length() == 0) {
                throw new InvalidXMLElementException("The " + attName + " attribute of " + e.getTagName()
                                                   + " element must be present and not empty");
            }
        }
        if (attValue != null && attValue.length() > 0) {
            return attValue;
        } else {
            return defaultValue;
        }
    }
  
}
