/*
 * Created on 11 nov. 2003
 *
 */
package com.jujunie.service.web;

/**
 * Represent a module in an application
 * @author <a href="mailto:julien@jujunie.com">julien</a>
 */
public interface WebModule {
    
    /**
     * @return the code of this module
     */
    String getCode();
    
    /**
     * @return the application which contains this module
     */
    WebApplication getApplication();
    
    /**
     * Get a webModule using a module path.
     * A module path is by example : "moduleCode1/moduleCode1.2/moduleCode1.2.5"
     * @param modulePath path of the module to retreive
     * @return the WebModule corresponding to the given path
     * @throws WebItemNotAvailableException if the requested module is not found
     */
    WebModule getModule(String modulePath) throws WebItemNotAvailableException;

    /**
     * Returns the page of this module corresponding to the given code
     * @param code code of the page to retreive
     * @return the page of this module corresponding to the given code
     * @throws WebItemNotAvailableException if the page connot be found
     */
    WebPage getPage(String code) throws WebItemNotAvailableException;
    
    /**
     * Returns the default page of this module
     * @return the default page of this module
     */
    WebPage getDefaultPage();
    
}
