/*
 * Created on Nov 29, 2003
 */
package com.jujunie.service.web;


/**
 * Represent a web page.
 * @author julien
 */
public interface WebPage {

    /** Default encoding */
    String DEFAULT_ENCODING = "UTF-8";
    
    /** Default content type */
    String DEFAULT_CONTENT_TYPE = "text/html";
    
    /** Get the encoding of the page */
    String getEncoding();
    
    /** Get content type */
    String getContentType();
    
    /**
     * @return the code of the page
     */
    String getCode();
    
    /**
     * The returned area is in the web area list which is the
     * template area list with some areas definition
     * overrided by the page.
     * @return the list of areas
     * @throws WebItemNotAvailableException if the reqested area is not found
     */
    WebArea getArea(String areaCode) throws WebItemNotAvailableException;
    
    /**
     * @return the template used by this page
     */
    WebTemplate getUsedTemplate();
    
    /**
     * Returns the Command corresponding to the given code
     * @param code Code of the command to retreive
     * @return the Command corresponding to the given code
     * @throws WebItemNotAvailableException if the reqested command is not found
     */
    Command getCommand(String code) throws WebItemNotAvailableException;
    
    /**
     * Returns the output WebPage corresponding to the given code
     * @param code Code of the output to retreive
     * @return the output WebPage corresponding to the given code
     * @throws WebItemNotAvailableException if the reqested command is not found
     */
    WebPage getOutput(String code) throws WebItemNotAvailableException;
    
    /**
     * Returns the display initialiser of this page.
     * @return the display initialiser of this page.
     */
    DisplayInitialiser getDisplayInitialiser();
    
    /**
     * Returns the display handler for this page
     * @return display handler
     */
    DisplayHandler getDisplayHandler();


    /**
     * @return application that contains this page
     */
    WebApplication getWebApplication();
    
}
