/*
 * Created on Nov 29, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.jujunie.service.web;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.jujunie.service.xml.*;

/**
 * This is an implementation of the WebTemplate interface
 * loaded using an XML DOM structure.
 * Nested into a application.
 * Required attributes :
 * <ul>
 *   <li>code : code of the template</li>
 *   <li>container : container of the structure defined by areas</li>
 * </ul>
 * Nested elements :
 * <ul>
 *   <li> area (0 to n). Defines the structure of the template See WebAreaXML</li>
 * </ul>
 * @see WebAreaXML
 * @author julien
 */
public class WebTemplateXML extends WebElementXML implements WebTemplate {

    /** serialVersionUID */
	private static final long serialVersionUID = 5400824917672837564L;

	/** Template element name */
    public static final String ELEMENT_NAME = "template";
    
    /** Template code */
    private String code = "";
    
    /** Template coontainer */
    private String container = "";
    
    /** Application linked to this template */
    private WebApplication application;
    
    /** Associated areas */
    private Map areas;
    
    /**
     * Loads associated areas
     * @param e the template element in the XML file
     * @throws InvalidXMLElementException
     */
    private void loadAreas(Element e) throws InvalidXMLElementException {
        NodeList areasList = e.getElementsByTagName(WebAreaXML.ELEMENT_NAME);
        WebArea current;
        this.areas = new HashMap(areasList.getLength());
        for (int i = 0; i < areasList.getLength(); i++) {
            current = new WebAreaXML(areasList.item(i));
            this.areas.put(current.getCode(), current);
        }
    }
    
    
    /**
     * Constructor
     * @param mod the XM node representing the template
     * @param app parent application
     * @throws InvalidXMLElementException
     */
    protected WebTemplateXML(Node tpl, WebApplication app) throws InvalidXMLElementException {
        Element tplElement;
        this.application = app;
        if (tpl.getNodeType() != Node.ELEMENT_NODE) {
            throw new InvalidXMLElementException("The given XML node is not an element");
        }
        tplElement = (Element) tpl;
        if (!tplElement.getTagName().equals(ELEMENT_NAME)) {
            throw new InvalidXMLElementException("The given Element is not a " + ELEMENT_NAME
                                               + " element");
        }
        this.code = super.getAttribute(tplElement, ATT_CODE, true);
        this.container = super.getAttribute(tplElement, ATT_CONTAINER, true);
        // Loading areas
        this.loadAreas(tplElement);
    }
    
    /* (non-Javadoc)
     * @see com.jujunie.service.web.WebTemplate#getCode()
     */
    public String getCode() {
        return this.code;
    }

    /* (non-Javadoc)
     * @see com.jujunie.service.web.WebTemplate#getArea(java.lang.String)
     */
    public WebArea getArea(String areaName) throws WebItemNotAvailableException {
        WebArea res = (WebArea) this.areas.get(areaName);
        if (res == null) {
            throw new WebItemNotAvailableException("The area named " + areaName + " not found");
        }
        return res;
    }


    /* (non-Javadoc)
     * @see com.jujunie.service.web.WebTemplate#getAreaList()
     */
    public WebAreaList getAreaList() {
        return new TemplateWebAreaList(this.areas.values().iterator());
    }


    /* (non-Javadoc)
     * @see com.jujunie.service.web.WebTemplate#getContainer()
     */
    public String getContainer() {
        return this.container;
    }

    /**
     * Private class used to implement the WebAreaList interface to the
     * area list of a template.
     * @author julien
     */
    private class TemplateWebAreaList implements WebAreaList {

        private Iterator i;
        
        /**
         * Creates a TemplateWebAreaList.
         * 
         * @param areaIterator
         */
        public TemplateWebAreaList(Iterator areaIterator) {
            this.i = areaIterator;
        }
        
        /* (non-Javadoc)
         * @see com.jujunie.service.web.WebAreaList#hasNext()
         */
        public boolean hasNext() {
            return this.i.hasNext();
        }

        /* (non-Javadoc)
         * @see com.jujunie.service.web.WebAreaList#next()
         */
        public WebArea next() throws WebItemNotAvailableException {
            return (WebArea) this.i.next();
        }
    }
}
