package com.jujunie.service.web;

import java.io.Serializable;

import javax.servlet.http.HttpServletRequest;

/**
 * A web tarnsaction handler is called at the very beginning of the web transaction,
 * and at the very end. It has been origiannly created to handle hibernate "transaction for request"
 * paradigm.
 * @author julien
 */
public interface WebTransactionHandler extends Serializable {

    final static WebTransactionHandler DEFAULT = new DefaultWebTransactionHandler();
    
    /**
     * Begins the transaction
     * @param req HTTP request
     */
    void begin(HttpServletRequest req);
    
    /**
     * Ends the transaction
     * @param req HTTP request
     */
    void end(HttpServletRequest req);
    
    /**
     * Error notification
     * @param req HTTP request
     * @param error the error
     */
    void onError(HttpServletRequest req, Throwable error);
    
    class DefaultWebTransactionHandler implements WebTransactionHandler {

        private DefaultWebTransactionHandler() {
        }

        public void begin(HttpServletRequest req) {
            //nothing
        }

        public void end(HttpServletRequest req) {
            // nothing
        }

        public void onError(HttpServletRequest req, Throwable error) {
            // nothing;
        }
        
    }
}
