/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.jujunie.service.web;

import com.jujunie.service.log.Log;
import com.jujunie.service.xml.InvalidXMLElementException;
import java.io.IOException;
import java.io.Serializable;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.w3c.dom.Element;

/**
 * This display handler look for an implementation of {@link Writer} using
 * the key {@link WriterDisplayHandlerXML#WRITER_KEY}, and call the
 * {@link Writer#write(java.io.Writer)} method using the response writer.
 * @author julien
 * @since 1.01.01
 */
public class WriterDisplayHandlerXML
    extends WebElementXML implements DisplayHandlerXML, Serializable {

    public final static String WRITER_KEY = 
            "com.jujunie.service.web.WriterDisplayHandlerXML#WRITER_KEY";
    /**
     * serialVersionUID
     */
    private static final long serialVersionUID = -8430259675117787739L;

    /** Code */
    private String code = "";
    
    public void init(Element displayHandlerNode) 
            throws WebConfigException, InvalidXMLElementException {
        this.code = super.getAttribute(displayHandlerNode, ATT_CODE, true);
        // Nothing to do, yet
    }

    public void display(
            WebPage page, HttpServletRequest req, HttpServletResponse resp)
            throws DisplayException {
        // Initialise the view
        page.getDisplayInitialiser().initDisplay(new HttpRequestDisplayContext(req), req);
        // Setting responce properties
        resp.setContentType(page.getContentType() + "; charset=" + page.getEncoding());
        resp.setCharacterEncoding(page.getEncoding());
        // Retreiving the POJO object
        Object pojo = req.getAttribute(WRITER_KEY);
        if(pojo instanceof Writer) {
            try {
                ((Writer) pojo).write(resp.getWriter());
            } catch (IOException e) {
                String message = 
                        "Got IO error while serializing object: " +
                        e.getMessage();
                Log.getInstance(this).error(message, e);
                throw new DisplayException(message, e);
            }
        } else {
            throw new DisplayException(
                    "Cannot find an Object in request using" +
                    " WriterDisplayHandlerXML#WRITER_KEY key");
        }
    }
    
    public static interface Writer {
        void write(java.io.Writer w) throws IOException;
    }
    /**
     * @see com.jujunie.service.web.DisplayHandlerXML#getCode() 
     */    public String getCode() {
        return this.code;
    }
}
