/**
 * 
 */
package com.jujunie.service.web.html;

import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * This class represent a list structure
 * @author julien
 *
 */
public class Table implements Serializable {

    /**
     * serialVersionUID
     */
    private static final long serialVersionUID = 100914032793702815L;
    
    /** List of Column Managers */
    private List managers = new LinkedList();
    
    /** Row manager */
    private RowManager rowManager = null;
    
    /** StringBuffer where tthe result is stored */
    private StringBuffer res = null;
    
    /** Name of this table */
    private String name = "";

    /** Table CSS Style class */
    private String styleClass = null;

    
    /**
     * Creates a new Table
     * @param voList list of VO to display
     * @param rm RowManager
     * @throws IllegalArgumentException if one of the given parameters is null.
     */
    protected Table(RowManager rm, String tableName) throws IllegalArgumentException {
        if (rm == null || tableName == null) {
            throw new IllegalArgumentException("Neither VO list or row manager can be null");
        }
        this.rowManager = rm;
        this.name = tableName;
    }
    
    /** Add a column manager */
    protected void addColumnManager(ColumnManager m) {
        this.managers.add(m);
    }
    
    /**
     * Add an hidden field
     * @param fieldName field name
     * @param fieldValue field value
     */
    protected void addHiddenField(String fieldName, String fieldValue) {
        this.res.append("<input type=\"hidden\" value=\"" + fieldValue + "\" name=\"" + fieldName + "\"/>");
    }
    
    /**
     * Generates the HTML table
     * @return HTML code
     */
    public StringBuffer generate(TableInformation info) {
        // Initializing line counter
        info.initCurrentLineNumber();
        this.res = new StringBuffer();
        this.initManagers();
        this.res.append("<table class=\"").append(this.styleClass).append("\"><tr>");
        this.generateHeaders(info);
        this.generateRows(info);
        this.res.append("</tr>");
        return this.res;
    }
    
    /**
     * Init the managers
     */
    private void initManagers() {
        // Heach manager represent a column
        Iterator it = this.managers.iterator();
        ColumnManager current  = null;
        while (it.hasNext()) {
            current = (ColumnManager) it.next();
            current.init(this);
        }
    }
    
    /**
     * Generating the table headers
     */
    private void generateHeaders(TableInformation info) {
        // Heach manager represent a column and know how to generate the header of this row
        Iterator it = this.managers.iterator();
        ColumnManager current  = null;
        while (it.hasNext()) {
            current = (ColumnManager) it.next();
            current.generateHeader(info, this.res);
        }
    }
    
    /**
     * Generate the rows of a table
     */
    private void generateRows(TableInformation info) {
        // Generating a row for each vo in the list of VO
        Iterator it = info.getVos().iterator();
        Iterator cols = null;
        ColumnManager col  = null;
        while (it.hasNext()) {
            this.res.append("<tr>");
            this.rowManager.setVO(it.next(), info.getCurrentLineNumber());
            info.increaseCurrentLineNumber();
            // Generating a cell for each column
            cols = this.managers.iterator();
            while (cols.hasNext()) {
                col = (ColumnManager) cols.next();
                this.rowManager.nextColumnNotification();
                col.generateCell(info, this.rowManager, this.res);
            }
            this.res.append("</tr>");
        }
    }

    /**
     * @return the name of this table
     */
    protected String getName() {
        return name;
    }

    protected String getStyleClass() {
        return styleClass;
    }

    protected void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

}
